from pycsp3.problems.data.parsing import *

data["mapSize"] = next_int()
nRadars = next_int()
data["maxCoverage"] = next_int()
nInsignificantCells = next_int()
series = next_int()
seed = next_int()


# TODO to be finished (as in JvSCP3)